/* ESP8266  Data Log NTP client Example  (ESP8266_03)
   shield_FE with JTX to D8, JRX to D9, CH_PD to 7
   data log  for temperature with LM35 on pin AN0
   compiled with Arduino 1.6.3
   testet with ESP8266-01 Firmware 0020000903
   tested with ESP8266-03 Firmware 0018000902
  
   Change SSID and PASS to match your WiFi settings.
   The IP address is displayed after the successful connection.   
*/   

const int ESP_RX = 8;
const int ESP_TX = 9;
const int ESP_PD = 7;
const int chipSelect = 4;

// If using Software Serial for debug use:
//#include <SoftwareSerial.h>
//SoftwareSerial Debug(8,9);  // use pins 8, 9 for software serial 
//#define ESP8266 Serial

// If your MCU has dual USARTs (e.g. ATmega644 or Leonardo) use:
//#define Debug Serial    // use Serial for debug
//#define ESP8266 Serial1   // use Serial1 to talk to esp8266

// If using Software Serial for comunicate with ESP use:
#include <SoftwareSerial.h>
SoftwareSerial ESP8266(ESP_RX, ESP_TX);  // use pins 8, 9 for software serial 
#define Debug Serial    // use Serial for debug


#include <Time.h> 
#include <SPI.h>
#include <SD.h>

#define DEBUG true
#define SSID        "SSID"
#define PASS        "PASSWORD"

unsigned long time = 0;
// buffer to hold incoming and outgoing NTP packets
const int NTP_PACKET_SIZE= 48; 
byte packetBuffer[NTP_PACKET_SIZE]; 
// buffer to hold incoming response from ESP8266
#define BUFFER_SIZE 512
char buffer[BUFFER_SIZE];
const unsigned long UTCcorrection = 7200;  //Time Zone + Daylight saving time correction 
   

unsigned long timeToLog=30000;  //30 sec Time to LOG
unsigned long timeActual;

void setup()
{ 
  pinMode(ESP_PD, INPUT); //pullup R set to high
  
  delay(1000);
  pinMode(chipSelect, OUTPUT);
  
  analogReference(INTERNAL); //1.1V Vref
  // open Debug Serial Port for communicate with PC
  Debug.begin(9600);  
  Debug.println("ESP8266 Shield_FE DataLog Demo");
  
  // Open Serial Port for communicate with EPS8266
  ESP8266.begin(9600); //it is the speed of ESP8266
  ESP8266.setTimeout(5000);  //timeout for receive response from ESP8266  

  // see if the card is present and can be initialized:
  if (!SD.begin(chipSelect))
  {
    Debug.println("Card failed, or not present");
    // don't do anything more:
    return;
  }
  Debug.println("card initialized.");
  
  //clear read buffer
      while(ESP8266.available())     
        char c = ESP8266.read(); // read the next character.  
  
  //test if the module is ready  
  ESP8266.println("AT+RST");
  delay(1000);
  if (ESP8266.find("OK"))
  {
    Debug.println("Module is ready");
  }
  else
  {
    Debug.println("Module have no response.");
    while (1);  //ALT execute
  }

  //clear read buffer
      while(ESP8266.available())     
        char c = ESP8266.read(); // read the next character. 
  
  sendData("AT+CIPMUX=0\r\n",1000,DEBUG); // configure for single connections  
  sendData("AT+CWMODE=1\r\n",1000,DEBUG); // connect to router
  sendData("ATE0\r\n",1000,DEBUG); // no ECHO
  
  // Sleep module
  pinMode(ESP_PD, OUTPUT); 
  digitalWrite(ESP_PD, LOW);
  setPacket();  //set packet for NTP request
}



void loop()
{
  timeActual=millis();
  
  Debug.println("\n\nStart new LOG");
  // WakeUp module
  pinMode(ESP_PD, INPUT); //pullup R set to high
  delay(2000);
  while (ESP8266.available())
      char c = ESP8266.read();
      
  // connect to wifi
  boolean connected = false;
  for (int i = 0; i < 5; i++)
  {
    if (connectWiFi())
    {
      connected = true;
      break;
    }
  }
  if (!connected) {
    return;
  }
  delay(1000);
  
  //request NTP Time
  requestTime();  
  
    unsigned long t=millis();
    int i=0;
    String response = "";        
    long int time = millis();    
    long int timeout = 5000;
    while( (time+timeout) > millis())
    {
      while(ESP8266.available())
      {        
        char c = ESP8266.read(); // read the next character.
        response+=c;
      }  
    }
    
    if (DEBUG)
    {
      Debug.print("Response->");
      Debug.println(response);
    }  

    int ind=response.indexOf("+IPD");
    if (ind>=0)
    {
      for(i=0;i<NTP_PACKET_SIZE;i++)
      packetBuffer[i]=response[ind+8+i];
      parsePackets();  
      Debug.println("OK");    
    }
    else
    {
//        Debug.println("Bad response from NTP");
    }
    
    //clear read buffer
      while(ESP8266.available())     
        char c = ESP8266.read(); // read the next character.  
  // disconnect and Sleep module
  sendData("AT+CWQAP\r\n",1000,DEBUG); // disconnect Wifi   
  pinMode(ESP_PD, OUTPUT); 
  digitalWrite(ESP_PD, LOW);
  Debug.println("disconnect.");
  
  // READ analog value
  String stringVal = "";     //data on buff is copied to this string  
  char charVal[4];
  float floatVal = float(analogRead(A0))/9.31;
  dtostrf(floatVal, 4, 1, charVal);
  for(i=0;i<sizeof(charVal);i++)
  {
    stringVal+=charVal[i];
  }
  String dataString = getDateString() + " " + getTimeString() + " " + stringVal +  char(176) + "C";  

  File dataFile = SD.open("datalog.txt", FILE_WRITE);

  // if the file is available, write to it:
  if (dataFile) 
  {
    dataFile.println(dataString);
    dataFile.close();
    // print to the serial port too:
    Debug.print("Write SD-> ");
    Debug.println(dataString);
  }  
  else 
  {
    Debug.println("error opening datalog.txt");
  }   
 
 
  //wait for new log
    while( (timeActual+timeToLog) > millis())
    {
      }
}
 

int requestTime()
{
   //clear read buffer
      while(ESP8266.available())     
        char c = ESP8266.read(); // read the next character. 
  
  //Connect to the NTP server  
  String cmd1 = "AT+CIPSTART=\"UDP\",\"";
  cmd1 += "129.6.15.28";
  cmd1 += "\",123";
  Debug.print("connect server:");
  Debug.println(cmd1);  
  ESP8266.println(cmd1);

  time = millis();
  while ((time + 5000) >= millis())
  {
     if (ESP8266.available() > 0)
       Debug.write(ESP8266.read());  
  } 
  
  // send packet
  ESP8266.println("AT+CIPSEND=48");
  // attendi ">"  
   if(ESP8266.find(">"))
   {
     if (DEBUG) Debug.println("found >");
   }
   else
   {
     ESP8266.println("AT+CIPCLOSE");
     Debug.println("connect timeout");
     return(0);
   }

  ESP8266.write(packetBuffer,NTP_PACKET_SIZE);

  if(ESP8266.find("SEND OK") ) 
  {
      Debug.println("SEND Request to NTP...");    
  }  
  else
      Debug.println("NO SEND");   

}

 
String sendData(String command, const int timeout, boolean debug)
{
    String response = "";  
    ESP8266.print(command); // send the read character to the ESP8266    
    long int time = millis();    
    while( (time+timeout) > millis())
    {
      while(ESP8266.available())
      {      
        // The esp has data so display its output to the serial window 
        char c = ESP8266.read(); // read the next character.
        response+=c;
      }  
    }
    
    if(DEBUG) Debug.print(response);    
    return response;
}


boolean connectWiFi()
{
  String response = "";
  String cmd = "AT+CWJAP=\"";
  cmd += SSID;
  cmd += "\",\"";
  cmd += PASS;
  cmd += "\"";
  if (DEBUG) Debug.println(cmd);
  ESP8266.println(cmd);
  delay(2000);  
  
  if (ESP8266.find("OK"))
  {
    Debug.println("OK, Connected to WiFi.");
    return true;
  } else
  {
    Debug.println("Can not connect to the WiFi.");
    return false;
  }
 

}

void setPacket(){
  // set all bytes in the buffer to 0
  memset(packetBuffer, 0, NTP_PACKET_SIZE); 
  // Initialize values needed to form NTP request
  packetBuffer[0] = 0b11100011;   // LI, Version, Mode
  packetBuffer[1] = 0;     // Stratum, or type of clock
  packetBuffer[2] = 6;     // Polling Interval
  packetBuffer[3] = 0xEC;  // Peer Clock Precision
  // 8 bytes of zero for Root Delay & Root Dispersion
  packetBuffer[12]  = 49; 
  packetBuffer[13]  = 0x4E;
  packetBuffer[14]  = 49;
  packetBuffer[15]  = 52;
}

void parsePackets() 
{  
    //the timestamp starts at byte 40 of the received packet and is four bytes,
    // or two words, long. First, esxtract the two words:

    unsigned long highWord = word(packetBuffer[40], packetBuffer[41]);
    unsigned long lowWord = word(packetBuffer[42], packetBuffer[43]);  
    // combine the four bytes (two words) into a long integer
    // this is NTP time (seconds since Jan 1 1900):
    unsigned long secsSince1900 = highWord << 16 | lowWord;  
//    Serial.println();
//    Serial.print("Seconds since Jan 1 1900 = " );
//    Serial.println(secsSince1900);               

    // now convert NTP time into everyday time:
    // Serial.print("Unix time = ");
    // Unix time starts on Jan 1 1970. In seconds, that's 2208988800:
    const unsigned long seventyYears = 2208988800UL;     
    // subtract seventy years:
    unsigned long epoch = secsSince1900 - seventyYears;  
    // print Unix time:
    // Serial.println(epoch);                               
    time_t t = epoch + UTCcorrection;  
    setTime(t);    // sets arduino internal clock 
    
//    Serial.println(getDateString());
//    Serial.println(getTimeString());
}

String getTimeString() {
  // gives back hh:mm:ss
  time_t t = now();
  String s = "";
  if (hour(t) <10) s = s + "0";
  s = s + hour(t) + ":";
  if (minute(t) <10) s = s + "0";
  s = s + minute(t) + ":";
  if (second(t) <10) s = s + "0";
  s = s + second(t);
  return(s);
}


String getDateString() {
  // gives back dd/mm/yyyy
  time_t t = now();
  String s = "";
  if (day(t) <10) s = s + "0";
  s = s + day(t) + "/";
  if (month(t) <10) s = s + "0";
  s = s + month(t) + "/";
  s = s + year(t);
  return(s);  
}
