/* ====== ESP8266 Web Server Demo ======
 * Print out analog value
 *
 * Change SSID and PASS to match your WiFi settings.
 * The IP address is displayed after the successful connection.
 *
 * Tested with Shield_FE and module ESP8266-03 
 * jump ESPRX->D9
 * jump ESPTX->D8
 * jump ESPPD-> not connect
 */

#define DEBUG true

#include <SoftwareSerial.h>
#define SSID        "SSID"
#define PASS        "PASS"
SoftwareSerial ESP8266(8, 9); // RX, TX  ESP8266

void setup()
{  
  // open Hardware Serial Port for communicate with PC
  Serial.begin(9600);  
  Serial.println("ESP8266 Demo");
  
  // Open Software Serial Port for communicate with EPS8266
  ESP8266.begin(9600); //it is the speed of ESP8266
  ESP8266.setTimeout(5000);  //timeout for receive response from ESP8266  
  
  //test if the module is ready
  ESP8266.println("AT+RST");
  delay(1000);
  if (ESP8266.find("OK"))
  {
    Serial.println("Module is ready");
  }
  else
  {
    Serial.println("Module have no response.");
    while (1);
  }
  delay(1000);
  //connect to the wifi
  boolean connected = false;
  for (int i = 0; i < 5; i++)
  {
    if (connectWiFi())
    {
      connected = true;
      break;
    }
  }
  if (!connected) {
    while (1);
  }
  delay(5000);
  //print the ip addr
  ESP8266.println("AT+CIFSR");  //request IP module ESP8266 assigned by DHCP
  Serial.println("ip address:");
  while (ESP8266.available())
    Serial.write(ESP8266.read());
  sendData("AT+CIPMUX=1\r\n",1000,DEBUG); // configure for multiple connections
  sendData("AT+CIPSERVER=1,80\r\n",1000,DEBUG); // turn on server on port 80
}



void loop()
{
  if(ESP8266.available()) // check if request 
  {
    
    if(ESP8266.find("+IPD,"))
    {
     delay(1000);
 
     int connectionId = ESP8266.read()-48; // subtract 48 because the read() function returns 
                                           // the ASCII decimal value and 0 (the first decimal number) starts at 48

     String webpage = "<title>Webserver</title><H3>Welcome to Web Server</H3><p>Value A0 = "+ String(analogRead(A0), DEC) +"</p>";
 
     String cipSend = "AT+CIPSEND=";
     cipSend += connectionId;
     cipSend += ",";
     cipSend +=webpage.length();
     cipSend +="\r\n";
     
     sendData(cipSend,1000,DEBUG);
     sendData(webpage,1000,DEBUG);
 
     String closeCommand = "AT+CIPCLOSE="; 
     closeCommand+=connectionId; // append connection id
     closeCommand+="\r\n";
     
     sendData(closeCommand,3000,DEBUG);
    }
  }
}
 
 
String sendData(String command, const int timeout, boolean debug)
{
    String response = "";
    
    ESP8266.print(command); // send the read character to the ESP8266
    
    long int time = millis();
    
    while( (time+timeout) > millis())
    {
      while(ESP8266.available())
      {
        
        // The esp has data so display its output to the serial window 
        char c = ESP8266.read(); // read the next character.
        response+=c;
      }  
    }
    
    if(debug)
    {
      Serial.print(response);
    }
    
    return response;
}

boolean connectWiFi()
{
  ESP8266.println("AT+CWMODE=1");
  String cmd = "AT+CWJAP=\"";
  cmd += SSID;
  cmd += "\",\"";
  cmd += PASS;
  cmd += "\"";
  Serial.println(cmd);
  ESP8266.println(cmd);
  delay(2000);
  if (ESP8266.find("OK"))
  {
    Serial.println("OK, Connected to WiFi.");
    return true;
  } else
  {
    Serial.println("Can not connect to the WiFi.");
    return false;
  }
}
